import { DatabaseQueryModel, PouchService } from "@justbe-angular/jb-pouch";
import { StoreService } from "@justbe-angular/jb-store";
import { ObjectUtilityService } from "./object-utility.service";
import { LoggerService } from "@justbe-angular/jb-pouch/services/logger.service";
import { Injectable } from "@angular/core";
var BizObjectService = (function () {
    function BizObjectService(pouchService, storeService, objectUtility, loggerService) {
        this.pouchService = pouchService;
        this.storeService = storeService;
        this.objectUtility = objectUtility;
        this.loggerService = loggerService;
    }
    ;
    BizObjectService.prototype.sync = function (configurations) {
        var _this = this;
        var obj = new DatabaseQueryModel();
        if (configurations.initialLoader && configurations.initialLoader.selector) {
            obj.selector = configurations.initialLoader.selector;
        }
        if (configurations.selector) {
            obj.selector = configurations.selector;
        }
        obj.databaseName = configurations.name;
        this.pouchService.queryDatabase(obj).subscribe(function (res) {
            if (res.length === 0) {
                _this.freshCall(configurations);
            }
            else if (res.length > 0) {
                if (res[0]) {
                    // ?this means i found previous record with id so need to fetch delta
                    // ?but before that we are setting current document
                    // ?this means i found previous record with id so need to fetch delta
                    // ?but before that we are setting current document
                    _this.storeService.updateSubstore(obj.databaseName, res[0]);
                    _this.deltaCall(configurations);
                }
            }
        }, function (err) {
            _this.loggerService.error([err, configurations]);
        });
    };
    ;
    BizObjectService.prototype.freshCall = function (configurations) {
        var _this = this;
        configurations.fresh.function.apply(this, configurations.fresh.params).subscribe(function (freshRes) {
            if (freshRes.id) {
                var duo = new Date().getTime();
                if (configurations.custom && configurations.custom.getDuo) {
                    duo = configurations.custom.getDuo(freshRes);
                }
                freshRes.duo = duo;
                if (!freshRes._id) {
                    freshRes._id = freshRes.id;
                }
                _this.pouchService.upsert(configurations.bizobject.parent, freshRes).subscribe(function (res) {
                    console.log(res);
                    console.log('i am into upsert fresh document');
                    if (res === 200) {
                        _this.storeService.insertIntoSubStore(configurations.bizobject.parent, freshRes);
                        _this.storeService.updateSubstore(configurations.bizobject.name, freshRes);
                    }
                }, function (err) {
                    _this.loggerService.error([err, configurations]);
                });
            }
            else {
                _this.loggerService.info(['nothing found in fresh call', configurations]);
            }
        });
    };
    BizObjectService.prototype.deltaCall = function (configurations) {
        // INFO:delta call here
        // no delta call is implemented right now
        // TODO:so i am directly saving to pouch
        // this.pouchService.upsert(configurations.name, res[0]).subscribe(res => {
        // }, err => {
        // })
    };
    BizObjectService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    BizObjectService.ctorParameters = function () { return [
        { type: PouchService, },
        { type: StoreService, },
        { type: ObjectUtilityService, },
        { type: LoggerService, },
    ]; };
    return BizObjectService;
}());
export { BizObjectService };
//# sourceMappingURL=biz-object.service.js.map