import { DatabaseQueryModel, PouchService } from "@justbe-angular/jb-pouch";
import { StoreService } from "@justbe-angular/jb-store";
import { ObjectUtilityService } from "./object-utility.service";
import { LoggerService } from "@justbe-angular/jb-pouch/services/logger.service";
import { Injectable } from "@angular/core";
var BizobjectService = (function () {
    function BizobjectService(pouchService, storeService, objectUtility, loggerService) {
        this.pouchService = pouchService;
        this.storeService = storeService;
        this.objectUtility = objectUtility;
        this.loggerService = loggerService;
    }
    ;
    BizobjectService.prototype.sync = function (configurations) {
        var _this = this;
        var obj = new DatabaseQueryModel();
        if (configurations.initialLoader && configurations.initialLoader.selector) {
            obj.selector = configurations.initialLoader.selector;
        }
        if (configurations.selector) {
            obj.selector = configurations.selector;
        }
        obj.databaseName = this.getDatabaseName(configurations);
        this.pouchService.queryDatabase(obj).subscribe(function (res) {
            if (res.length === 0) {
                _this.freshCall(configurations);
            }
            else if (res.length > 0) {
                if (res[0]) {
                    // ?this means i found previous record with id so need to fetch delta
                    // ?but before that we are setting current document
                    // ?this means i found previous record with id so need to fetch delta
                    // ?but before that we are setting current document
                    _this.storeService.updateSubstore(configurations.bizobject.name, res[0]);
                    _this.storeService.insertIntoSubStore(configurations.bizobject.parent, res[0]);
                    _this.deltaCall(configurations);
                }
            }
        }, function (err) {
            _this.loggerService.error([err, configurations]);
        });
    };
    ;
    BizobjectService.prototype.freshCall = function (configurations) {
        var _this = this;
        configurations.fresh.function.apply(this, configurations.fresh.params).subscribe(function (freshRes) {
            if (freshRes.id) {
                var duo = new Date().getTime();
                if (configurations.custom && configurations.custom.getDuo) {
                    duo = configurations.custom.getDuo(freshRes);
                }
                freshRes.duo = duo;
                if (!freshRes._id) {
                    freshRes._id = freshRes.id;
                }
                if (!configurations.bizobject.parent) {
                    _this.pouchService.upsertDocument(configurations.bizobject.name, freshRes).subscribe(function (res) {
                        if (res === 200) {
                            _this.storeService.updateSubstore(configurations.bizobject.name, freshRes);
                        }
                    }, function (err) {
                        _this.loggerService.error([err, configurations]);
                    });
                }
                else {
                    _this.pouchService.upsertDocument(configurations.bizobject.parent, freshRes).subscribe(function (res) {
                        if (res === 200) {
                            _this.storeService.insertIntoSubStore(configurations.bizobject.parent, freshRes);
                            _this.storeService.updateSubstore(configurations.bizobject.name, freshRes);
                        }
                    }, function (err) {
                        _this.loggerService.error([err, configurations]);
                    });
                }
            }
            else {
                _this.loggerService.info(['nothing found in fresh call', configurations]);
            }
        });
    };
    BizobjectService.prototype.deltaCall = function (configurations) {
        this.loggerService.info([
            'biz object delta is not implemented yet',
            'hence calling fresh call again'
        ]);
        this.freshCall(configurations);
    };
    BizobjectService.prototype.getDatabaseName = function (configurations) {
        if (configurations.bizobject && configurations.bizobject.name) {
            if (!configurations.bizobject.parent) {
                return configurations.bizobject.name;
            }
            else {
                return configurations.bizobject.parent;
            }
        }
    };
    BizobjectService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    BizobjectService.ctorParameters = function () { return [
        { type: PouchService, },
        { type: StoreService, },
        { type: ObjectUtilityService, },
        { type: LoggerService, },
    ]; };
    return BizobjectService;
}());
export { BizobjectService };
//# sourceMappingURL=bizobject.service.js.map