import { Injectable } from "@angular/core";
import { Observable } from "rxjs";
import { PouchService } from "@justbe-angular/jb-pouch";
import { StoreService } from '@justbe-angular/jb-store';
import { LoggerService } from "@justbe-angular/jb-pouch/services/logger.service";
var ObjectHandlerService = (function () {
    function ObjectHandlerService(pouchService, storeService, logger) {
        this.pouchService = pouchService;
        this.storeService = storeService;
        this.logger = logger;
    }
    ;
    ObjectHandlerService.prototype.handle = function (objectToBehandled, configurations) {
        var _this = this;
        return new Observable(function (observer) {
            if (configurations.custom && configurations.custom.getObjectTobeSaved) {
                objectToBehandled = configurations.custom.getObjectTobeSaved(objectToBehandled);
            }
            if (objectToBehandled) {
                _this.saveObject(objectToBehandled, configurations).subscribe(function (SaveObjRes) {
                    var selector = _this.getSelector(configurations);
                    if (selector) {
                        var keys = Object.keys(selector);
                        var trueResults = new Array();
                        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
                            var key = keys_1[_i];
                            if (selector[key] === objectToBehandled[key]) {
                                trueResults.push(true);
                            }
                        }
                        if (keys.length === trueResults.length) {
                            _this.storeService.insertIntoSubStore(configurations.name, objectToBehandled);
                            observer.next(200);
                            observer.complete();
                        }
                    }
                    else {
                        _this.storeService.insertIntoSubStore(configurations.name, objectToBehandled);
                        observer.next(200);
                        observer.complete();
                    }
                }, function (saveObjErr) {
                    observer.error(saveObjErr);
                });
            }
            else {
                observer.error(404);
                _this.logger.info(['nothing found in object delta call']);
            }
        });
    };
    ObjectHandlerService.prototype.saveObject = function (objectToBehandled, configurations) {
        if (!objectToBehandled._id) {
            objectToBehandled._id = objectToBehandled.id;
        }
        // objectToBehandled.duo = new Date().getTime();
        return this.pouchService.upsertDocument(configurations.name, objectToBehandled).map(function (res) {
            console.log(res);
        }, function (err) {
            console.log(err);
        });
    };
    ObjectHandlerService.prototype.getSelector = function (configurations) {
        if (configurations.selector) {
            return configurations.selector;
        }
        else if (configurations.initialLoader.selector) {
            return configurations.initialLoader.selector;
        }
    };
    ObjectHandlerService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    ObjectHandlerService.ctorParameters = function () { return [
        { type: PouchService, },
        { type: StoreService, },
        { type: LoggerService, },
    ]; };
    return ObjectHandlerService;
}());
export { ObjectHandlerService };
//# sourceMappingURL=object-handler.service.js.map